'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../../../constants');
}

var _componentMap;

function _load_componentMap() {
  return _componentMap = _interopRequireDefault(require('../../../bit-map/component-map'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../../bit-id');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../../component'));
}

var _driver;

function _load_driver() {
  return _driver = require('../../../../driver');
}

var _utils;

function _load_utils() {
  return _utils = require('../../../../utils');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../../../logger/logger'));
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../../../../consumer');
}

var _dependencies;

function _load_dependencies() {
  return _dependencies = _interopRequireDefault(require('../dependencies'));
}

var _generalError;

function _load_generalError() {
  return _generalError = _interopRequireDefault(require('../../../../error/general-error'));
}

var _;

function _load_() {
  return _ = require('..');
}

var _envExtension;

function _load_envExtension() {
  return _envExtension = _interopRequireDefault(require('../../../../extensions/env-extension'));
}

var _bitMap;

function _load_bitMap() {
  return _bitMap = _interopRequireDefault(require('../../../bit-map'));
}

var _linkContent;

function _load_linkContent() {
  return _linkContent = require('../../../../links/link-content');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DependencyResolver = function () {
  function DependencyResolver(component, consumer, componentId) {
    _classCallCheck(this, DependencyResolver);

    this.component = component;
    this.consumer = consumer;
    this.componentId = componentId;
    this.componentMap = this.component.componentMap;
    this.componentFromModel = this.component.componentFromModel;
    this.consumerPath = this.consumer.getPath();
    this.allDependencies = {
      dependencies: [],
      devDependencies: [],
      compilerDependencies: [],
      testerDependencies: []
    };
    this.allPackagesDependencies = {
      packageDependencies: {},
      devPackageDependencies: {},
      compilerPackageDependencies: {},
      testerPackageDependencies: {}
    };
    this.processedFiles = [];
    this.issues = {
      missingPackagesDependenciesOnFs: {},
      missingComponents: {},
      untrackedDependencies: {},
      missingDependenciesOnFs: {},
      missingLinks: {},
      missingCustomModuleResolutionLinks: {},
      relativeComponents: {},
      parseErrors: {},
      resolveErrors: {},
      missingBits: {} // temporarily, will be combined with missingComponents. see combineIssues
    };
  }

  _createClass(DependencyResolver, [{
    key: 'setTree',
    value: function setTree(tree) {
      this.tree = tree;
    }

    /**
     * Load components and packages dependencies for a component. The process is as follows:
     * 1) Use the language driver to parse the component files and find for each file its dependencies.
     * 2) The results we get from the driver per file tells us what are the files and packages that depend on our file.
     * and also whether there are missing packages and files.
     * 3) Using the information from the driver, we go over each one of the dependencies files and find its counterpart
     * component. The way how we find it, is by using the bit.map file which has a mapping between the component name and
     * the file paths.
     * 4) If we find a component to the file dependency, we add it to component.dependencies. Otherwise, it's added to
     * component.issues.untrackedDependencies
     * 5) Similarly, when we find the packages dependencies, they are added to component.packageDependencies. Otherwise,
     * they're added to component.issues.missingPackagesDependenciesOnFs
     * 6) In case the driver found a file dependency that is not on the file-system, we add that file to
     * component.issues.missingDependenciesOnFs
     */

  }, {
    key: 'loadDependenciesForComponent',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(bitDir) {
        var _this = this;

        var driver, _componentMap$getFile, nonTestsFiles, testsFiles, allFiles, getDependenciesTree, dependenciesTree;

        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                driver = this.consumer.driver;
                _componentMap$getFile = this.componentMap.getFilesGroupedByBeingTests(), nonTestsFiles = _componentMap$getFile.nonTestsFiles, testsFiles = _componentMap$getFile.testsFiles;

                this.setCompilerFiles();
                this.setTesterFiles();
                allFiles = [].concat(_toConsumableArray(nonTestsFiles), _toConsumableArray(testsFiles), _toConsumableArray(this.compilerFiles), _toConsumableArray(this.testerFiles));

                getDependenciesTree = function () {
                  var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee() {
                    return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            return _context.abrupt('return', driver.getDependencyTree(bitDir, _this.consumerPath, allFiles, _this.component.bindingPrefix, _this.consumer.bitJson.resolveModules));

                          case 1:
                          case 'end':
                            return _context.stop();
                        }
                      }
                    }, _callee, _this);
                  }));

                  return function getDependenciesTree() {
                    return _ref2.apply(this, arguments);
                  };
                }();
                // find the dependencies (internal files and packages) through automatic dependency resolution


                _context2.next = 8;
                return getDependenciesTree();

              case 8:
                dependenciesTree = _context2.sent;

                // we have the files dependencies, these files should be components that are registered in bit.map. Otherwise,
                // they are referred as "untracked components" and the user should add them later on in order to commit
                this.setTree(dependenciesTree.tree);
                this.populateDependencies(allFiles, testsFiles);
                this.component.setDependencies(this.allDependencies.dependencies);
                this.component.setDevDependencies(this.allDependencies.devDependencies);
                this.component.setCompilerDependencies(this.allDependencies.compilerDependencies);
                this.component.setTesterDependencies(this.allDependencies.testerDependencies);
                this.component.packageDependencies = this.allPackagesDependencies.packageDependencies;
                this.component.devPackageDependencies = this.allPackagesDependencies.devPackageDependencies;
                this.component.compilerPackageDependencies = (_ramda || _load_ramda()).default.merge(this.allPackagesDependencies.compilerPackageDependencies, this.component.compilerPackageDependencies);
                this.component.testerPackageDependencies = (_ramda || _load_ramda()).default.merge(this.allPackagesDependencies.testerPackageDependencies, this.component.testerPackageDependencies);
                this.component.peerPackageDependencies = findPeerDependencies(this.consumerPath, this.component);
                if (!(_ramda || _load_ramda()).default.isEmpty(this.issues)) this.component.issues = this.issues;

                return _context2.abrupt('return', this.component);

              case 22:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function loadDependenciesForComponent(_x) {
        return _ref.apply(this, arguments);
      }

      return loadDependenciesForComponent;
    }()

    /**
     * Given the tree of file dependencies from the driver, find the components of these files.
     * Each dependency file has a path, use bit.map to search for the component name by that path.
     * If the component is found, add it to "this.allDependencies.dependencies". Otherwise, add it to "this.issues.untrackedDependencies".
     *
     * For the found components, add their sourceRelativePath and destinationRelativePath, they are being used for
     * generating links upon import:
     * sourceRelativePath - location of the link file.
     * destinationRelativePath - destination written inside the link file.
     */

  }, {
    key: 'populateDependencies',
    value: function populateDependencies(files, testsFiles) {
      var _this2 = this;

      files.forEach(function (file) {
        var fileType = {
          isTestFile: (_ramda || _load_ramda()).default.contains(file, testsFiles),
          isCompilerFile: (_ramda || _load_ramda()).default.contains(file, _this2.compilerFiles),
          isTesterFile: (_ramda || _load_ramda()).default.contains(file, _this2.testerFiles)
        };
        if (!_this2.tree[file]) {
          throw new Error(`DependencyResolver: a file "${file}" was not returned from the driver, its dependencies are unknown`);
        }
        _this2.processMissing(file);
        _this2.processErrors(file);
        _this2.processPackages(file, fileType);
        _this2.processBits(file, fileType);
        _this2.processDepFiles(file, fileType);
        _this2.processUnidentifiedPackages(file, fileType);
      });
      this.removeDevAndEnvDepsIfTheyAlsoRegulars();
      this.copyEnvDependenciesFromModelIfNeeded();
      this.combineIssues();
      this.removeEmptyIssues();
    }
  }, {
    key: 'traverseTreeForComponentId',
    value: function traverseTreeForComponentId(depFile) {
      if (!this.tree[depFile] || !this.tree[depFile].files && !this.tree[depFile].bits) return;
      if (!this.componentMap.rootDir) {
        throw Error('traverseTreeForComponentId should get called only when rootDir is set');
      }
      var rootDirFullPath = (_path || _load_path()).default.join(this.consumerPath, this.componentMap.rootDir);
      if (this.tree[depFile].files && this.tree[depFile].files.length) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = this.tree[depFile].files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var file = _step.value;

            var fullDepFile = (_path || _load_path()).default.resolve(rootDirFullPath, file.file);
            var depRelativeToConsumer = (0, (_utils || _load_utils()).pathNormalizeToLinux)((_path || _load_path()).default.relative(this.consumerPath, fullDepFile));
            var componentId = this.consumer.bitMap.getComponentIdByPath(depRelativeToConsumer);
            if (componentId) return componentId; // eslint-disable-line consistent-return
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }
      if (this.tree[depFile].bits && this.tree[depFile].bits.length) {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = this.tree[depFile].bits[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var bit = _step2.value;

            var _componentId = this.consumer.getComponentIdFromNodeModulesPath(bit, this.component.bindingPrefix);
            if (_componentId) return _componentId; // eslint-disable-line consistent-return
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }

      if (this.tree[depFile].files && this.tree[depFile].files.length) {
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = this.tree[depFile].files[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _file = _step3.value;

            if (_file.file !== depFile) {
              var _componentId2 = this.traverseTreeForComponentId(_file.file);
              if (_componentId2) return _componentId2; // eslint-disable-line consistent-return
            } else {
              (_logger || _load_logger()).default.warn(`traverseTreeForComponentId found a cyclic dependency. ${_file.file} depends on itself`);
            }
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      }
    }
  }, {
    key: 'getComponentIdByDepFile',
    value: function getComponentIdByDepFile(depFile) {
      var depFileRelative = depFile; // dependency file path relative to consumer root
      var componentId = void 0;
      var destination = void 0;
      var rootDir = this.componentMap.rootDir;
      if (rootDir) {
        // The depFileRelative is relative to rootDir, change it to be relative to current consumer.
        // We can't use path.resolve(rootDir, fileDep) because this might not work when running
        // bit commands not from root, because resolve take by default the process.cwd
        var rootDirFullPath = (_path || _load_path()).default.join(this.consumerPath, rootDir);
        var fullDepFile = (_path || _load_path()).default.resolve(rootDirFullPath, depFile);
        depFileRelative = (0, (_utils || _load_utils()).pathNormalizeToLinux)((_path || _load_path()).default.relative(this.consumerPath, fullDepFile));
      }

      componentId = this.consumer.bitMap.getComponentIdByPath(depFileRelative);
      // if not found here, the file is not a component file. It might be a bit-auto-generated file.
      // find the component file by the auto-generated file.
      // We make sure also that rootDir is there, otherwise, it's an AUTHORED component, which shouldn't have
      // auto-generated files.
      if (!componentId && rootDir) {
        componentId = this.traverseTreeForComponentId(depFile);
        if (componentId) {
          // it is verified now that this depFile is an auto-generated file, therefore the sourceRelativePath and the
          // destinationRelativePath should be a partial-path and not full-relative-to-consumer path.
          // since the dep-file is a generated file, it is safe to assume that the componentFromModel has in its
          // dependencies array this component with the relativePaths array. Find the relativePath of this dep-file
          // to get the correct destinationRelativePath. There is no other way to obtain this info.
          if (!this.componentFromModel) {
            throw new (_generalError || _load_generalError()).default(`Failed to resolve ${componentId.toString()} dependencies because the component is not in the model.
Try to run "bit import ${this.component.id.toString()} --objects" to get the component saved in the model`);
          }

          var _getDependencyPathsFr = this.getDependencyPathsFromModel(componentId, depFile, rootDir);

          componentId = _getDependencyPathsFr.componentId;
          destination = _getDependencyPathsFr.destination;
          depFileRelative = _getDependencyPathsFr.depFileRelative;
        } else if (!(0, (_linkContent || _load_linkContent()).isSupportedExtension)(depFile) && this.componentFromModel) {
          // unsupported files, such as binary files, don't have link files. instead, they have a
          // symlink (or sometimes a copy on Windows) of the dependency inside the component. to
          // check whether a file is a symlink to a dependency we loop through the
          // sourceRelativePaths of the dependency, if there is match, we use the data from the model
          var dependenciesFromModel = this.componentFromModel.getAllDependenciesCloned();
          var sourcePaths = dependenciesFromModel.getSourcesPaths();
          if (sourcePaths.includes(depFile)) {
            var dependencyFromModel = dependenciesFromModel.getBySourcePath(depFile);
            componentId = dependencyFromModel.id;

            var _getDependencyPathsFr2 = this.getDependencyPathsFromModel(componentId, depFile, rootDir);

            componentId = _getDependencyPathsFr2.componentId;
            destination = _getDependencyPathsFr2.destination;
            depFileRelative = _getDependencyPathsFr2.depFileRelative;
          }
        }
      }

      return { componentId, depFileRelative, destination };
    }
  }, {
    key: 'getDependencyPathsFromModel',
    value: function getDependencyPathsFromModel(componentId, depFile, rootDir) {
      var dependency = this.componentFromModel.getAllDependencies() // $FlowFixMe
      .find(function (dep) {
        return dep.id.isEqualWithoutVersion(componentId);
      });
      if (!dependency) {
        throw new (_generalError || _load_generalError()).default( // $FlowFixMe
        `the auto-generated file ${depFile} should be connected to ${componentId}, however, it's not part of the model dependencies of ${this.componentFromModel.id}`);
      }
      var isCompilerDependency = this.componentFromModel.compilerDependencies.getById(componentId);
      var isTesterDependency = this.componentFromModel.testerDependencies.getById(componentId);
      var isEnvDependency = isCompilerDependency || isTesterDependency;
      var isRelativeToConfigDir = Boolean(isEnvDependency && this.componentMap.configDir && this.componentMap.rootDir);
      var originallySource = this.getOriginallySourcePath(isRelativeToConfigDir, rootDir, depFile, isCompilerDependency);
      var relativePath = dependency.relativePaths.find(function (r) {
        return r.sourceRelativePath === originallySource;
      });
      if (!relativePath) {
        throw new (_generalError || _load_generalError()).default(`unable to find ${originallySource} path in the dependencies relativePaths of ${this.componentFromModel.id}`);
      }
      return {
        componentId: dependency.id,
        destination: relativePath.destinationRelativePath,
        // in the case of isRelativeToConfigDir, sourceRelativePath should be relative to configDir
        depFileRelative: isRelativeToConfigDir ? originallySource : depFile
      };
    }
  }, {
    key: 'getOriginallySourcePath',
    value: function getOriginallySourcePath(isRelativeToConfigDir, rootDir, depFile, isCompilerDependency) {
      if (isRelativeToConfigDir) {
        // find the sourceRelativePath relative to the configDir, not to the rootDir of the component
        var resolvedSource = (_path || _load_path()).default.resolve(rootDir, depFile);
        // @todo: use the new ConfigDir class that Gilad added once it is merged.

        var _BitMap$parseConfigDi = (_bitMap || _load_bitMap()).default.parseConfigDir(this.componentMap.configDir, this.componentMap.rootDir),
            compilerConfigDir = _BitMap$parseConfigDi.compiler,
            testerConfigDir = _BitMap$parseConfigDi.tester;

        var configDir = isCompilerDependency ? compilerConfigDir : testerConfigDir;
        var absoluteConfigDir = this.consumer.toAbsolutePath(configDir);
        return (0, (_utils || _load_utils()).pathRelativeLinux)(absoluteConfigDir, resolvedSource);
      }
      return depFile;
    }
  }, {
    key: 'processDepFiles',
    value: function processDepFiles(originFile, fileType) {
      var _this3 = this;

      var allDepsFiles = this.tree[originFile].files;
      if (!allDepsFiles || (_ramda || _load_ramda()).default.isEmpty(allDepsFiles)) return;
      allDepsFiles.forEach(function (depFile) {
        if (depFile.isLink) _this3.processLinkFile(originFile, depFile, fileType);else {
          _this3.processOneDepFile(originFile, depFile.file, depFile.importSpecifiers, undefined, fileType, depFile);
        }
      });
    }
  }, {
    key: 'processOneDepFile',
    value: function processOneDepFile(originFile, depFile, importSpecifiers, linkFile, fileType, depFileObject) {
      if (this.processedFiles.includes(depFile)) return;
      this.processedFiles.push(depFile);

      // if the dependency of an envFile is already included in the env files of the component, we're good
      if (fileType.isCompilerFile && this.compilerFiles.includes(depFile)) return;
      if (fileType.isTesterFile && this.testerFiles.includes(depFile)) return;

      var _getComponentIdByDepF = this.getComponentIdByDepFile(depFile),
          componentId = _getComponentIdByDepF.componentId,
          depFileRelative = _getComponentIdByDepF.depFileRelative,
          destination = _getComponentIdByDepF.destination;
      // the file dependency doesn't have any counterpart component. Add it to this.issues.untrackedDependencies


      if (!componentId) {
        if (this.issues.untrackedDependencies[originFile]) {
          this.issues.untrackedDependencies[originFile].push(depFileRelative);
        } else {
          this.issues.untrackedDependencies[originFile] = [depFileRelative];
        }
        return;
      }

      // happens when in the same component one file requires another one. In this case, there is
      // noting to do regarding the dependencies
      if (componentId.isEqual(this.componentId)) {
        if (depFileObject.isCustomResolveUsed) {
          this.component.customResolvedPaths.push({
            destinationPath: depFileObject.file,
            importSource: depFileObject.importSource
          });
        }
        return;
      }

      var depComponentMap = this.consumer.bitMap.getComponentIfExist(componentId);
      // found a dependency component. Add it to this.allDependencies.dependencies
      var depRootDir = depComponentMap ? depComponentMap.rootDir : undefined;
      var destinationRelativePath = destination || (depRootDir && depFileRelative.startsWith(depRootDir) ? (0, (_utils || _load_utils()).pathRelativeLinux)(depRootDir, depFileRelative) : depFileRelative);

      var sourceRelativePath = void 0;
      if (linkFile) {
        sourceRelativePath = linkFile;
      } else {
        // when there is no rootDir for the current dependency (it happens when it's AUTHORED), keep the original path
        sourceRelativePath = depRootDir ? depFileRelative : depFile;
      }

      var depsPaths = {
        sourceRelativePath,
        destinationRelativePath
      };
      if (importSpecifiers) {
        depsPaths.importSpecifiers = importSpecifiers;
      }
      if (depFileObject.isCustomResolveUsed) {
        depsPaths.isCustomResolveUsed = depFileObject.isCustomResolveUsed;
        depsPaths.importSource = depFileObject.importSource;
      }
      var currentComponentsDeps = { id: componentId, relativePaths: [depsPaths] };

      if (depComponentMap && (depComponentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED && this.componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED || depComponentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED && this.componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED)) {
        // prevent author using relative paths for IMPORTED component
        // also prevent adding AUTHORED component to an IMPORTED component using relative syntax. The reason is that when
        // this component is imported somewhere else, a link-file between the IMPORTED and the AUTHORED must be written
        // outside the component directory, which might override user files.
        if (this.issues.relativeComponents[originFile]) this.issues.relativeComponents[originFile].push(componentId);else this.issues.relativeComponents[originFile] = [componentId];
        return;
      }

      var allDependencies = [].concat(_toConsumableArray(this.allDependencies.dependencies), _toConsumableArray(this.allDependencies.devDependencies), _toConsumableArray(this.allDependencies.compilerDependencies), _toConsumableArray(this.allDependencies.testerDependencies));
      var existingDependency = this.getExistingDependency(allDependencies, componentId);
      if (existingDependency) {
        // it is another file of an already existing component. Just add the new path
        existingDependency.relativePaths.push(depsPaths);
      } else {
        this.pushToDependenciesArray(currentComponentsDeps, fileType);
      }
    }
  }, {
    key: 'processLinkFile',
    value: function processLinkFile(originFile, linkFile, fileType) {
      var _this4 = this;

      if (!linkFile.linkDependencies || (_ramda || _load_ramda()).default.isEmpty(linkFile.linkDependencies)) return;
      var nonLinkImportSpecifiers = [];
      linkFile.linkDependencies.forEach(function (dependency) {
        var component = _this4.getComponentIdByDepFile(linkFile.file);
        if (component.componentId) {
          // the linkFile is already a component, no need to treat it differently than other depFile
          // aggregate all dependencies using the same linkFile and ultimately run processDepFile
          // with all importSpecifiers of that linkFile.
          // also, delete the linkFile attribute of importSpecifiers so then once the component is
          // imported and the link is generated, it won't be treated as a linkFile.
          dependency.importSpecifiers.map(function (a) {
            return delete a.linkFile;
          });
          nonLinkImportSpecifiers.push(dependency.importSpecifiers);
        } else {
          _this4.processOneDepFile(originFile, dependency.file, dependency.importSpecifiers, linkFile.file, fileType, linkFile);
        }
      });
      if (nonLinkImportSpecifiers.length) {
        this.processOneDepFile(originFile, linkFile.file, (_ramda || _load_ramda()).default.flatten(nonLinkImportSpecifiers), undefined, fileType, linkFile);
      }
    }
  }, {
    key: 'processBits',
    value: function processBits(originFile, fileType) {
      var _this5 = this;

      var bits = this.tree[originFile].bits;
      if (!bits || (_ramda || _load_ramda()).default.isEmpty(bits)) return;
      bits.forEach(function (bitDep) {
        var componentId = _this5.consumer.getComponentIdFromNodeModulesPath(bitDep, _this5.component.bindingPrefix);
        var getExistingId = function getExistingId() {
          var existingId = _this5.consumer.bitmapIds.searchWithoutVersion(componentId);
          if (existingId) return existingId;

          // maybe the dependencies were imported as npm packages
          if (bitDep.startsWith('node_modules')) {
            // Add the root dir in case it exists (to make sure we search for the dependency package json in the correct place)
            var basePath = _this5.componentMap.rootDir ? (_path || _load_path()).default.join(_this5.consumerPath, _this5.componentMap.rootDir) : _this5.consumerPath;
            var depPath = (_path || _load_path()).default.join(basePath, bitDep);
            var packageJson = _this5.consumer.driver.driver.PackageJson.findPackage(depPath);
            if (packageJson) {
              var depVersion = packageJson.version;
              existingId = componentId.changeVersion(depVersion);
              return existingId;
            }
          }
          if (_this5.componentFromModel) {
            var modelDep = _this5.componentFromModel.getAllDependenciesIds().searchWithoutVersion(componentId);
            if (modelDep) return modelDep;
          }
          return null;
        };
        var existingId = getExistingId();
        if (existingId) {
          var currentComponentsDeps = { id: existingId, relativePaths: [] };
          var existingDependency = _this5.getExistingDependency(_this5.allDependencies.dependencies, existingId);
          if (!existingDependency) {
            _this5.pushToDependenciesArray(currentComponentsDeps, fileType);
          }
        } else {
          _this5.issues.missingBits[originFile] ? _this5.issues.missingBits[originFile].push(componentId) : _this5.issues.missingBits[originFile] = [componentId];
        }
      });
    }
  }, {
    key: 'processPackages',
    value: function processPackages(originFile, fileType) {
      var packages = this.tree[originFile].packages;
      if (packages && !(_ramda || _load_ramda()).default.isEmpty(packages)) {
        if (fileType.isTestFile) {
          Object.assign(this.allPackagesDependencies.devPackageDependencies, packages);
        } else if (fileType.isCompilerFile) {
          Object.assign(this.allPackagesDependencies.compilerPackageDependencies, packages);
        } else if (fileType.isTesterFile) {
          Object.assign(this.allPackagesDependencies.testerPackageDependencies, packages);
        } else {
          Object.assign(this.allPackagesDependencies.packageDependencies, packages);
        }
      }
    }
  }, {
    key: 'processMissing',
    value: function processMissing(originFile) {
      var _this6 = this;

      var missing = this.tree[originFile].missing;
      if (!missing) return;
      if (missing.files && !(_ramda || _load_ramda()).default.isEmpty(missing.files)) {
        if (this.issues.missingDependenciesOnFs[originFile]) {
          this.issues.missingDependenciesOnFs[originFile].concat(missing.files);
        } else this.issues.missingDependenciesOnFs[originFile] = missing.files;
      }
      if (missing.packages && !(_ramda || _load_ramda()).default.isEmpty(missing.packages)) {
        var customResolvedDependencies = this.findOriginallyCustomModuleResolvedDependencies(missing.packages);
        if (customResolvedDependencies) {
          Object.keys(customResolvedDependencies).forEach(function (missingPackage) {
            var componentId = customResolvedDependencies[missingPackage].toString();
            if (_this6.issues.missingCustomModuleResolutionLinks[originFile]) {
              _this6.issues.missingCustomModuleResolutionLinks[originFile].push(componentId);
            } else _this6.issues.missingCustomModuleResolutionLinks[originFile] = [componentId];
          });
        }
        var missingPackages = customResolvedDependencies ? (_ramda || _load_ramda()).default.difference(missing.packages, Object.keys(customResolvedDependencies)) : missing.packages;

        if (!(_ramda || _load_ramda()).default.isEmpty(missingPackages)) {
          if (this.issues.missingPackagesDependenciesOnFs[originFile]) {
            this.issues.missingPackagesDependenciesOnFs[originFile].concat(missing.packages);
          } else this.issues.missingPackagesDependenciesOnFs[originFile] = missing.packages;
        }
      }

      if (missing.bits && !(_ramda || _load_ramda()).default.isEmpty(missing.bits)) {
        missing.bits.forEach(function (missingBit) {
          var componentId = _this6.consumer.getComponentIdFromNodeModulesPath(missingBit, _this6.component.bindingPrefix);
          // todo: a component might be on bit.map but not on the FS, yet, it's not about missing links.
          if (_this6.consumer.bitMap.getBitIdIfExist(componentId, { ignoreVersion: true })) {
            if (_this6.issues.missingLinks[originFile]) _this6.issues.missingLinks[originFile].push(componentId);else _this6.issues.missingLinks[originFile] = [componentId];
          } else if (_this6.issues.missingComponents[originFile]) {
            _this6.issues.missingComponents[originFile].push(componentId);
          } else {
            _this6.issues.missingComponents[originFile] = [componentId];
          }
        });
      }
    }
  }, {
    key: 'processErrors',
    value: function processErrors(originFile) {
      var error = this.tree[originFile].error;
      if (!error) return;
      (_logger || _load_logger()).default.error('got an error from the driver while resolving dependencies');
      (_logger || _load_logger()).default.error(error);
      // $FlowFixMe error.code is set when it comes from bit-javascript, otherwise, it's undefined and treated as resolve-error
      if (error.code === 'PARSING_ERROR') this.issues.parseErrors[originFile] = error.message;else this.issues.resolveErrors[originFile] = error.message;
    }

    /**
     * currently the only unidentified packages being process are the ones coming from custom-modules-resolution.
     * assuming the author used custom-resolution, which enable using non-relative import syntax,
     * for example, requiring the file 'src/utils/is-string' from anywhere as require('utils/is-string');
     * now, when the component is imported, the driver recognizes 'utils/is-string' as a package,
     * because it's not relative.
     * the goal here is to use the 'package' the driver found and match it with one of the
     * dependencies from the model. In the example above, we might find in the model, a dependency
     * is-string with importSource of 'utils/is-string'.
     * Once a match is found, copy the relativePaths from the model.
     */

  }, {
    key: 'processUnidentifiedPackages',
    value: function processUnidentifiedPackages(originFile, fileType) {
      var _this7 = this;

      var unidentifiedPackages = this.tree[originFile].unidentifiedPackages;
      if (!unidentifiedPackages) return;
      if (!this.componentFromModel) return; // not relevant, the component is not imported
      var getDependencies = function getDependencies() {
        if (fileType.isTestFile) return _this7.componentFromModel.devDependencies;
        if (fileType.isCompilerFile) return _this7.componentFromModel.compilerDependencies;
        if (fileType.isTesterFile) return _this7.componentFromModel.testerDependencies;
        return _this7.componentFromModel.dependencies;
      };
      var dependencies = getDependencies();
      if (dependencies.isEmpty()) return;
      var importSourceMap = dependencies.getCustomResolvedData();
      if ((_ramda || _load_ramda()).default.isEmpty(importSourceMap)) return;
      var clonedDependencies = new (_dependencies || _load_dependencies()).default(dependencies.getClone());
      unidentifiedPackages.forEach(function (unidentifiedPackage) {
        var packageLinuxFormat = (0, (_utils || _load_utils()).pathNormalizeToLinux)(unidentifiedPackage);
        var packageWithNoNodeModules = packageLinuxFormat.replace('node_modules/', '');
        var foundImportSource = Object.keys(importSourceMap).find(function (importSource) {
          return packageWithNoNodeModules.startsWith(importSource);
        });
        if (foundImportSource) {
          var dependencyId = importSourceMap[foundImportSource];
          var existingDependency = _this7.getExistingDependency(_this7.allDependencies.dependencies, dependencyId);
          var existingDevDependency = _this7.getExistingDependency(_this7.allDependencies.devDependencies, dependencyId);
          var existingCompilerDependency = _this7.getExistingDependency(_this7.allDependencies.compilerDependencies, dependencyId);
          var existingTesterDependency = _this7.getExistingDependency(_this7.allDependencies.testerDependencies, dependencyId);
          var currentComponentDeps = {
            id: dependencyId,
            relativePaths: clonedDependencies.getById(dependencyId).relativePaths
          };
          if (fileType.isTestFile && !existingDevDependency) {
            _this7.allDependencies.devDependencies.push(currentComponentDeps);
          } else if (fileType.isCompilerFile && !existingCompilerDependency) {
            _this7.allDependencies.compilerDependencies.push(currentComponentDeps);
          } else if (fileType.isTesterFile && !existingTesterDependency) {
            _this7.allDependencies.testerDependencies.push(currentComponentDeps);
          } else if (!fileType.isTestFile && !fileType.isCompilerFile && !fileType.isTesterFile && !existingDependency) {
            _this7.allDependencies.dependencies.push(currentComponentDeps);
          }
        }
      });
    }
  }, {
    key: 'pushToDependenciesArray',
    value: function pushToDependenciesArray(currentComponentsDeps, fileType) {
      if (fileType.isTestFile) {
        this.allDependencies.devDependencies.push(currentComponentsDeps);
      } else if (fileType.isCompilerFile) {
        this.allDependencies.compilerDependencies.push(currentComponentsDeps);
      } else if (fileType.isTesterFile) {
        this.allDependencies.testerDependencies.push(currentComponentsDeps);
      } else {
        this.allDependencies.dependencies.push(currentComponentsDeps);
      }
    }

    /**
     * Remove the dependencies which appear both in dev and regular deps from the dev
     * Because if a dependency is both dev dependency and regular dependency it should be treated as regular one
     * Apply for both packages and components dependencies
     */

  }, {
    key: 'removeDevAndEnvDepsIfTheyAlsoRegulars',
    value: function removeDevAndEnvDepsIfTheyAlsoRegulars() {
      var _this8 = this;

      // remove dev and env packages that are also regular packages
      var getNotRegularPackages = function getNotRegularPackages(packages) {
        return (_ramda || _load_ramda()).default.difference((_ramda || _load_ramda()).default.keys(packages), (_ramda || _load_ramda()).default.keys(_this8.allPackagesDependencies.packageDependencies));
      };
      this.allPackagesDependencies.devPackageDependencies = (_ramda || _load_ramda()).default.pick(getNotRegularPackages(this.allPackagesDependencies.devPackageDependencies), this.allPackagesDependencies.devPackageDependencies);
      this.allPackagesDependencies.compilerPackageDependencies = (_ramda || _load_ramda()).default.pick(getNotRegularPackages(this.allPackagesDependencies.compilerPackageDependencies), this.allPackagesDependencies.compilerPackageDependencies);
      this.allPackagesDependencies.testerPackageDependencies = (_ramda || _load_ramda()).default.pick(getNotRegularPackages(this.allPackagesDependencies.testerPackageDependencies), this.allPackagesDependencies.testerPackageDependencies);

      // remove dev and env dependencies that are also regular dependencies
      var componentDepsIds = new (Function.prototype.bind.apply((_bitId || _load_bitId()).BitIds, [null].concat(_toConsumableArray(this.allDependencies.dependencies.map(function (c) {
        return c.id;
      })))))();
      this.allDependencies.devDependencies = this.allDependencies.devDependencies.filter(function (d) {
        return !componentDepsIds.has(d.id);
      });
      this.allDependencies.compilerDependencies = this.allDependencies.compilerDependencies.filter(function (d) {
        return !componentDepsIds.has(d.id);
      });
      this.allDependencies.testerDependencies = this.allDependencies.testerDependencies.filter(function (d) {
        return !componentDepsIds.has(d.id);
      });
    }

    /**
     * given missing packages name, find whether they were dependencies with custom-resolve before.
     */

  }, {
    key: 'findOriginallyCustomModuleResolvedDependencies',
    value: function findOriginallyCustomModuleResolvedDependencies(packages) {
      if (!packages) return undefined;
      if (!this.componentFromModel) return undefined; // not relevant, the component is not imported
      var dependencies = new (_dependencies || _load_dependencies()).default(this.componentFromModel.getAllDependencies());
      if (dependencies.isEmpty()) return undefined;
      var importSourceMap = dependencies.getCustomResolvedData();
      if ((_ramda || _load_ramda()).default.isEmpty(importSourceMap)) return undefined;
      var foundPackages = packages.reduce(function (acc, value) {
        var packageLinuxFormat = (0, (_utils || _load_utils()).pathNormalizeToLinux)(value);
        var foundImportSource = Object.keys(importSourceMap).find(function (importSource) {
          return importSource.startsWith(packageLinuxFormat);
        });
        if (foundImportSource) {
          var dependencyId = importSourceMap[foundImportSource];
          acc[value] = dependencyId;
        }
        return acc;
      }, {});

      return (_ramda || _load_ramda()).default.isEmpty(foundPackages) ? undefined : foundPackages;
    }

    // when env dependencies are not sent to the dependency-resolver, they should be copied from the model

  }, {
    key: 'copyEnvDependenciesFromModelIfNeeded',
    value: function copyEnvDependenciesFromModelIfNeeded() {
      if (!this.componentFromModel) return;
      if (!shouldProcessEnvDependencies(this.component.compiler)) {
        this.allDependencies.compilerDependencies = this.componentFromModel.compilerDependencies.get();
        this.allPackagesDependencies.compilerPackageDependencies = this.componentFromModel.compilerPackageDependencies;
      }
      if (!shouldProcessEnvDependencies(this.component.tester)) {
        this.allDependencies.testerDependencies = this.componentFromModel.testerDependencies.get();
        this.allPackagesDependencies.testerPackageDependencies = this.componentFromModel.testerPackageDependencies;
      }
    }
  }, {
    key: 'combineIssues',
    value: function combineIssues() {
      var _this9 = this;

      Object.keys(this.issues.missingBits).forEach(function (missingBit) {
        if (_this9.issues.missingComponents[missingBit]) {
          _this9.issues.missingComponents[missingBit] = _this9.issues.missingComponents[missingBit].concat(_this9.issues.missingBits[missingBit]);
        } else {
          _this9.issues.missingComponents[missingBit] = _this9.issues.missingBits[missingBit];
        }
      });
    }
  }, {
    key: 'removeEmptyIssues',
    value: function removeEmptyIssues() {
      var notEmpty = function notEmpty(item) {
        return !(_ramda || _load_ramda()).default.isEmpty(item);
      };
      this.issues = (_ramda || _load_ramda()).default.filter(notEmpty, this.issues);
    }
  }, {
    key: 'getExistingDependency',
    value: function getExistingDependency(dependencies, id) {
      return dependencies.find(function (d) {
        return d.id.isEqual(id);
      });
    }
  }, {
    key: 'setCompilerFiles',
    value: function setCompilerFiles() {
      var compilerFiles = shouldProcessEnvDependencies(this.component.compiler) ? this.component.compiler.files : [];
      this.compilerFiles = this.getRelativeEnvFiles(compilerFiles);
    }
  }, {
    key: 'setTesterFiles',
    value: function setTesterFiles() {
      var testerFiles = shouldProcessEnvDependencies(this.component.tester) ? this.component.tester.files : [];
      this.testerFiles = this.getRelativeEnvFiles(testerFiles);
    }
  }, {
    key: 'getRelativeEnvFiles',
    value: function getRelativeEnvFiles(files) {
      var _this10 = this;

      var getPathsRelativeToComponentRoot = function getPathsRelativeToComponentRoot() {
        var rootDirAbsolute = _this10.consumer.toAbsolutePath(_this10.componentMap.rootDir || '.');
        return files.map(function (file) {
          var envAbsolute = file.path;
          return (_path || _load_path()).default.relative(rootDirAbsolute, envAbsolute);
        });
      };
      return getPathsRelativeToComponentRoot().map(function (file) {
        return (0, (_utils || _load_utils()).pathNormalizeToLinux)(file);
      });
    }
  }]);

  return DependencyResolver;
}();

// @todo: move to bit-javascript
/**
 * For author, the peer-dependencies are set in the root package.json
 * For imported components, we don't want to change the peerDependencies of the author, unless
 * we're certain the user intent to do so. Therefore, we ignore the root package.json and look for
 * the package.json in the component's directory.
 */


exports.default = DependencyResolver;
function findPeerDependencies(consumerPath, component) {
  var componentMap = component.componentMap;
  var getPeerDependencies = function getPeerDependencies() {
    var componentRoot = componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED ? consumerPath : componentMap.rootDir;
    var packageJsonLocation = (_path || _load_path()).default.join(componentRoot, 'package.json');
    if ((_fsExtra || _load_fsExtra()).default.existsSync(packageJsonLocation)) {
      try {
        var packageJson = (_fsExtra || _load_fsExtra()).default.readJsonSync(packageJsonLocation);
        if (packageJson.peerDependencies) return packageJson.peerDependencies;
      } catch (err) {
        (_logger || _load_logger()).default.error(`Failed reading the project package.json at ${packageJsonLocation}. Error Message: ${err.message}`);
      }
    }
    if (component.componentFromModel && componentMap.origin !== (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) {
      return component.componentFromModel.peerPackageDependencies;
    }
    return {};
  };
  var projectPeerDependencies = getPeerDependencies();
  var peerPackages = {};
  if ((_ramda || _load_ramda()).default.isEmpty(projectPeerDependencies)) return {};
  // check whether the peer-dependencies was actually require in the code. if so, remove it from
  // the packages/dev-packages and add it as a peer-package.
  Object.keys(projectPeerDependencies).forEach(function (pkg) {
    ['packageDependencies', 'devPackageDependencies'].forEach(function (field) {
      if (Object.keys(component[field]).includes(pkg)) {
        delete component[field][pkg];
        peerPackages[pkg] = projectPeerDependencies[pkg];
      }
    });
  });
  return peerPackages;
}

/**
 * if the component doesn't have the env files written on the filesystem there is nothing to pass
 * to the dependencyResolver
 */
function shouldProcessEnvDependencies(env) {
  return Boolean(env && env.files && env.files.every(function (file) {
    return !file.fromModel;
  }));
}